<?php

namespace Drupal\paragraph_cleanup\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Controller for listing paragraph types and managing orphaned paragraphs.
 */
class ParagraphCleanupController extends ControllerBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new BulkUpdateUserForm.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
    );
  }

  /**
   * Lists paragraph types used in content and identifies orphaned paragraphs.
   */
  public function listParagraphs() {
    $header = [
      $this->t('Paragraph Type'),
      $this->t('Usage Count'),
      $this->t('Delete'),
    ];

    $rows = [];
    $paragraph_types = $this->entityTypeManager->getStorage('paragraphs_type')->loadMultiple();

    foreach ($paragraph_types as $paragraph_type) {
      $type_id = $paragraph_type->id();
      $type_label = $paragraph_type->label();
      $usage_count = $this->getParagraphUsageCount($type_id);

      // Only list paragraph types used less than 5 times.
      if ($usage_count <= 5) {
        $orphan_count = $this->getOrphanedParagraphCount($type_id);

        $orphan_link = $orphan_count > 0 ? Link::fromTextAndUrl($this->t('Delete'), $this->getDeleteOrphansUrl($type_id))->toString() : $orphan_count;

        $rows[] = [
          $type_label,
          $usage_count,
          $orphan_link,
        ];
      }
    }

    return [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => $this->t('No paragraph types found with less than 5 usages.'),
    ];
  }

  /**
   * Get the usage count of a paragraph type.
   *
   * @param string $type_id
   *   The paragraph type ID.
   *
   * @return int
   *   The number of times the paragraph type is used.
   */
  protected function getParagraphUsageCount($type_id) {
    $query = $this->entityTypeManager->getStorage('paragraph')->getQuery()
      ->condition('type', $type_id)
      ->accessCheck(FALSE);

    return $query->count()->execute();
  }

  /**
   * Get the count of orphaned paragraphs of a specific type.
   *
   * @param string $type_id
   *   The paragraph type ID.
   *
   * @return int
   *   The number of orphaned paragraphs.
   */
  protected function getOrphanedParagraphCount($type_id) {
    $query = $this->entityTypeManager->getStorage('paragraph')->getQuery()
      ->condition('type', $type_id)
      ->accessCheck(TRUE)
      ->exists('parent_id', FALSE);
    return $query->count()->execute();
  }

  /**
   * Get the URL to delete orphaned paragraphs of a specific type.
   *
   * @param string $type_id
   *   The paragraph type ID.
   *
   * @return \Drupal\Core\Url
   *   The URL to delete orphaned paragraphs.
   */
  protected function getDeleteOrphansUrl($type_id) {
    return Url::fromRoute('paragraph_cleanup.delete_orphans', ['type_id' => $type_id]);
  }

  /**
   * Deletes orphaned paragraphs of a specific type.
   */
  public function deleteOrphans($type_id) {
    $query = $this->entityTypeManager->getStorage('paragraph')->getQuery()
      ->condition('type', $type_id)
      ->exists('parent_id', FALSE)
      ->accessCheck(FALSE);

    $orphan_ids = $query->execute();

    if (!empty($orphan_ids)) {
      $storage = $this->entityTypeManager->getStorage('paragraph');
      $paragraphs = $storage->loadMultiple($orphan_ids);
      $storage->delete($paragraphs);
      $this->messenger()->addMessage($this->t('Deleted @count orphaned paragraphs.', ['@count' => count($paragraphs)]));
    }
    else {
      $this->messenger()->addWarning($this->t('No orphaned paragraphs found for this type.'));
    }

    return $this->redirect('paragraph_cleanup.list_paragraphs');
  }

}
